###
   Copyright 2014+ Aliaksandr Litskevich, github.com/alitskevich

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

###

###

AXOID. A general-purpose micro-framework that just organizes code as a composition of Entities.

###

"use strict"

###

    Calls `fn` in next loop tick.
    
    @param [int] ms delay in millisec. Optional. Default is 5ms.
    @param [Object] ctx function context object. Optional. Default is null.
    @param [Function] fn function to call. Required.
    
    @example
        Function.nextTick(100, @, -> doing())

###
Function.nextTick = (ms, ctx, fn) ->

    ((fn = ms) and (ms = null) and (ctx = null)) if arguments.length is 1

    ((fn = ctx) and (ctx = ms) and (ms = null)) if arguments.length is 2

    setTimeout((-> fn.call ctx), ms or 5)


###

    Manages execution of sequence of functions allowing async and parallel flow.    

    @param [Object] ctx function context object. Optional. Default is null.
    @param [Function] factory function to return operations list. Required.
    
    @example
        Function.perform(@, (flow)-> [
            -> 
                r= doSome1()
                #next op
                flow.next(null, r)
            
            (err, r)-> 
                #async with callback
                doSomeAsync(r, flow.next)
            
            (err, data)-> 
                #parallel flow
                doSome1(d, flow.cb()) for d in data
                # next op
                flow.next(null, 1)
                
            (err, p1, results...)->
                #done
                
            ]

###
Function.perform = (ctx, factory) ->

    # flag for parallel flow sync
    locked = 1

    # arguments for next op
    results = [null, undefined]

    # calls next operation
    tick = () ->

        if not --locked and (op = flow.operations.shift())

            locked = 1

            _args = [].concat results
            results = [null, undefined]

            try
                # invoke current operation
                op.apply(flow.context, _args)

            catch ex
                Object.error(('perform: '+op)[0..150].replace(/\s+/g,' ')).addDetails(ex).log()

    # creates callback
    newCb = (pos)->

        (err, v) ->

            return unless (results[pos] is undefined) # can affect only once!
            results[0]   = Object.error(err).log() if err
            results[pos] = v
            tick()

    # flow controller accessible in ops closures
    flow =
        # a context object passed with each op call
        context: ctx
        # used to show a correct error stack
        _error: new Error()
        # to be used as callback to go to next operation
        next: newCb(1)
        # to be invoked to create a new callback for parralel flow
        wait: ->
            pos = results.length
            # ensure size
            results[pos] = undefined
            # increment locked
            locked+=1
            newCb(pos)

    # instantiate ops with `flow` passed in closure
    flow.operations = factory.call(ctx, flow)
    # start with first operation
    tick()

# creates a new function from source `code`
Function.create = (code, params) ->
    
    if code then Object.parse("function (#{if params then params.join ', ' else ''}){ return #{code}; }") else null

# iterates with `fn` over array or object keys
Function.iterate = (fn, obj, ctx, opts) ->

    if obj

      ln = obj.length
      if ln is +ln
        fn.call ctx, x, i, opts for x, i in obj
      else
        fn.call ctx, x, n, opts for own n, x of obj

    ctx

###
Objects.
###

# updates `obj` with own key/values from `extra`.
Object.update = (obj, extra) ->

    if obj and extra
      obj[n] = extra[n] for own n of extra

    obj

# get/set value of `obj` property by `key` in deep.
Object.prop = (obj, key, val) ->

    return null unless obj

    p = -1

    if arguments.length>2
        #setter mode
        (obj = (obj[k = key[p1..p-1]] or (obj[k] = {}))) while (p = key.indexOf(".", p1 = p + 1)) > -1
        obj[key[p1..]] = val

    else
        #getter mode
        (obj = obj[key[p1..p-1]]) while obj and (p = key.indexOf(".", p1 = p + 1)) > -1
        obj?[key[p1..]]

# safely clones `obj` and updates result with `delta`.
Object.clone = (obj, delta) ->

    return null unless obj

    r = new (obj.constructor)()
    r[n] = obj[n] for own n of obj
    r[n] = delta[n] for own n of delta if delta
    r

# returns object evaluated from given `s`.
Object.parse = (s, opts) ->

    return null unless s

    try
        Function.call(Function, "return #{s}").call(opts?.ctx)

    catch ex

        Object.error("bad-code: Object.parse: #{ex.message}").addDetails(s).log().end(null)
        
#overrides methods.
Object.update.mixin = (T, fn, args...) ->

    return T unless fn

    for own n, m of fn.apply(null, [_super = {}].concat(args))
        _super[n] = T[n] or ->
        T[n] = m
    T

###
Strings.
###

# current language
String.LANGUAGE = 'en'

# returns capitalized `s`
String.capitalize = (s) ->

    if s?.length then s[0].toUpperCase() + s[1..] else ""

# returns camelized `s`
String.camelize = (s, sep='_') ->

    if s?.length then ((if i then String.capitalize(t) else t) for t,i in s.split(sep)).join('') else ''

# Returns string formatted by `s`-template filled with rest of arguments.
String.format = (s, args...) ->

    if s then s.replace /\{(\d+)\}/g, ((s,d) -> args[+d] or '') else ''

# returns localized `s`
String.localize = ((_cache) ->

    $ = (s, l=String.LANGUAGE) ->

        if s then _cache[l]?[s] or String.capitalize(String.camelize(s)) else ""

    $.get = (s, l=String.LANGUAGE) ->

        if s then _cache[l]?[s] or null else null

    $.add = (l, delta) ->

        ((delta = l) and (l = null)) if arguments.length is 1
        l=String.LANGUAGE unless l
        Object.update _cache[l] or (_cache[l] = {}), delta

    $.asEnum = (s, l)->

        return null unless r=@get(s,l)

        for k,v of r when id = +k or k
            if typeof v is 'object'
                v.id = id
                v
            else
                id:id, name: ''+v

    $
    
)({})

String.tokenize= (s='', re = /\W/g, op)->

    pastLastIndex = 0
    while e = re.exec(s)
        #interval text
        op text if e.index and (text = s[pastLastIndex..e.index-1])
        op e[0], true
        pastLastIndex = re.lastIndex

    #rest
    op text if (text = s[pastLastIndex..])

###

    Uri class represents Uri as structure parsed from string.

    Uri grammar: [type:]//host[/path...][?{(key=value)...}][#hash]
    
    @example usage
    
        uri = Object.Uri.parse "db://update/doc1?force=true#version2"
    
###
class Object.Uri

    constructor: (@params = {}) ->

    #flag to easy check Uri instance
    isUri: true

    # set parameters
    setParams: (all)->
        @params = all
        @

    # update parameters
    updateParams: (delta)->
        Object.update @params, delta
        @

    # represent as string
    toString: ->
        r = (if @host then (if @type then "#{@type}://#{@host}/" else "//#{@host}/") else '')  + @path.join('/')

        sep='?'
        for own n,v of @params
            r += sep + n + "=" + encodeURIComponent(v)
            (sep = '&') if sep is '?'

        r += "#" + @hash if @hash

        r

    # create a new Uri instance parsed from string
    @parse: (s, params) ->

        return s if s?.isUri

        r = new Uri(params)

        return r unless s

        s = "" + s  unless s.split

        # extract type:
        if (p = s.indexOf("://")) > -1
            r.type = s[0..p-1]
            s = s[p+1..]

        # extract hash:
        if (p = s.indexOf("#")) > -1
            r.hash = s[p+1..]
            s = s[0..p-1]

        # extract query:
        [s, r.query] = s.split("?")

        (r.params[p[0]] = decodeURIComponent p[1]) for v in r.query.split "&" when (p = v.split "=").length > 1 if r.query

        r.id = s

        # work with path:
        p = s.split("/")
        if p[0] is ""
            p.shift()
            if p[0] is ""
                p.shift()
                r.host = p.shift()
        r.path = p

        r

#   @method #Object.error(err, details).
#   @return [Err] object
Object.error = (->

    class Err

        constructor:  (e) ->
            @details = []

            if typeof e is 'string'
                @reason = e.split(':')[0] if e
                @message = e if e
            else
                @reason = e.reason if e.reason
                @message = e.message if e.message

        reason: 'unknown'
        message: ''
        isError: true

        addDetails: () ->
            for det in arguments when det
                @details.unshift(det)
                @stack = det.stack if det.stack
            @

        addPrefix: (p)->
            @prefix = ''+p if p
            @
            
        # logs this
        log: ()->
            Object.log (@)
            @

        end: (x) -> x

        toString: ->
            "#{@prefix or ''} #{@message}"

        printIntoLog: (c) ->
            details = if @details?length is 1 then @details[0] else @details
            stack = @stack or (new Error).stack
            c.error @toString(), details, stack

    (err, details) ->
        (if err?.isError then err else new Err(err)).addDetails(details)

)()

# log stub.
Object.log = (x) -> x
